      subroutine vffan(e,ne,pe,bu,nb,pb,u,n,p,ip,gwtb,mass,wdth)
c----------------------------------------------------------------------c
c Returns matrix elements with the anomalous Wtb coupling 
c see K. Kolodziej, Phys. Lett. B584 (2004) 89, hep-ph/0312168,
c Input: 
c        e -  polarization vector,
c        bu - spinor,
c        pb - four momentum of bu,
c        pe - four momentum of incoming e,
c        nb,ne,n - numbers of polarization states,
c        ip=+1/-1 for u representing a particle/an antiparticle
c        if ip=+1 => Wtb coupling is given by gwtb(1:2,1:2), Eq. (4)
c        if ip=-1 => Wtb coupling is given by gwtb(1:2,3:4), Eq. (5)
c Output: u - fermion array,
c         p - its momentum.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 9 Feb., 2011
c 11 April, 2011: error in a(2,2,1) corrected
c 6 May, 2013: Complex mass of the top quark transfered through 
c the anomalous Wtb coupling.
c----------------------------------------------------------------------c

      implicit none

      integer nb,ne,n,ip

      integer, parameter :: dp=kind(1.d0)

      real(dp) :: mass,wdth
      real(dp), dimension(0:3) :: pb,pe,p
      complex(dp) :: gwtb(2,5)
      complex(dp), dimension(4,nb) :: bu
      complex(dp), dimension(0:3,ne) :: e
      complex(dp), dimension(4,n) :: u

c Local variables:

      integer ib,ie,i

      complex(dp), parameter :: icom=(0.d0,1.d0)
      complex(dp) :: f1l,f1r,f2l,f2r,mf,mf2,den
      complex(dp), dimension(4) :: u1,pp,pm
      complex(dp), dimension(2,2) :: b,c,ae,de
      complex(dp), dimension(0:3,2,2) :: a,d

      if (ip == 1) then
       f1r=gwtb(1,1)
       f1l=gwtb(2,1)
       f2r=gwtb(1,2)
       f2l=gwtb(2,2)
       mf2=gwtb(1,5)
      else if (ip == -1) then
       f1r=gwtb(1,3)
       f1l=gwtb(2,3)
       f2r=gwtb(1,4)
       f2l=gwtb(2,4)
       mf2=gwtb(2,5)
      else
       write(*,*)'fefan.f: ip =',ip,' is not allowed'
       stop
      end if

      if (wdth == 0.d0) then
       mf2=cmplx(mass**2,-mass*wdth,dp)
      end if

      mf=sqrt(mf2)

      p=pb+pe

      pp=ip*(/cmplx(p(0)-p(3),0.d0,dp),   ! (1,1)
     &        cmplx(-p(1),-p(2),dp),      ! (2,1)
     &        cmplx(-p(1),p(2),dp),       ! (1,2)
     &        cmplx(p(0)+p(3),0.d0,dp) /) ! (2,2)

      pm=(/pp(4),-pp(2),-pp(3),pp(1)/)

      den=pp(1)*pp(4)-pp(3)*pp(2)-mf2

      a(0,1,:)=(/cmplx(pe(3),0.d0,dp),cmplx(pe(1),-pe(2),dp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),dp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.d0,pe(2),dp),cmplx(pe(0)-pe(3),0.d0,dp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.d0,dp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.d0,-pe(1),dp),cmplx(0.d0,-pe(0)+pe(3),dp)/)
      a(2,2,:)=(/cmplx(0.d0,pe(0)+pe(3),dp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.d0,dp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      i=0

      do ib=1,nb
       do ie=1,ne
        b(1,:)=(/e(0,ie)-e(3,ie),-e(1,ie)+icom*e(2,ie)/)
        b(2,:)=(/-e(1,ie)-icom*e(2,ie),e(0,ie)+e(3,ie)/)
        c(1,:)=(/b(2,2),-b(1,2)/)
        c(2,:)=(/-b(2,1),b(1,1)/)
        ae=a(0,:,:)*e(0,ie)-a(1,:,:)*e(1,ie)
     &    -a(2,:,:)*e(2,ie)-a(3,:,:)*e(3,ie)
        de=d(0,:,:)*e(0,ie)-d(1,:,:)*e(1,ie)
     &    -d(2,:,:)*e(2,ie)-d(3,:,:)*e(3,ie)
        u1(:2)=f2l*matmul(ae,bu(:2,ib))+f1r*matmul(b,bu(3:,ib))
        u1(3:)=f1l*matmul(c,bu(:2,ib))+f2r*matmul(de,bu(3:,ib))
        i=i+1
        u(:,i)=(/mf*u1(1)+pp(1)*u1(3)+pp(3)*u1(4),
     &           mf*u1(2)+pp(2)*u1(3)+pp(4)*u1(4),
     &           mf*u1(3)+pm(1)*u1(1)+pm(3)*u1(2),
     &           mf*u1(4)+pm(2)*u1(1)+pm(4)*u1(2)/)
       end do
      end do

      u=u/den

      end




